% myapp.m -- demo application.
%
%      myapp
%
% A demo application using MTk and Coma.
%
% See also: mtk, coma

%% myapp.m --- the preceding comment is the documentation string.

% Copyright (C) 2008 Ralph Schleicher

% This file is part of MTk, the Matlab widget toolkit.

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the
%      distribution.
%
%    * Neither the name of the copyright holder nor the names of its
%      contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
% FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
% COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
% ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

%% Code:

% Program entry point.
function myapp

  % Create graphical user interface.
  f = mtk('figure', 0, '0x0', ...
          'Name', 'MyApp', ...
          'CloseRequestFcn', {coma('handle', 'event'), 'exit'}, ...
          'Resize', 'off', ...
          'Visible', 'off');

  mtk('menuitem', f, ...
      {'File', ...
       {'&Open...^o', coma('handle', 'event'), 'open'}, ...
       {'Close^w', coma('handle', 'event'), 'close'}, ...
       '--', ...
       {'&Save^s', coma('handle', 'event'), 'save'}, ...
       {'Save as...', coma('handle', 'event'), 'save as'}, ...
       '--', ...
       {'Exit', coma('handle', 'event'), 'exit'}});

  mtk('render', f, ...
      {'vbox' ...
       'Margin' 16 ...
       'Padding' 8 ...
       {'hbox' ...
        'Padding' 2 ...
        'Alignment' 'baseline' ...
        {'text' ...
         'String' 'Name:'} ...
        {'edit' ...
         'Flexibility' 1 ...
         'Tag' 'name'}} ...
       {'hbox' ...
        'Padding' 2 ...
        'Alignment' 'baseline' ...
        {'text' ...
         'String' 'Sex:'} ...
        {'popupmenu' ...
         'Flexibility' 1 ...
         'Tag' 'sex' ...
         'String' {'', 'male', 'female'} ...
         'UserData' {' ', 'm', 'f'}}} ...
       {'checkbox' ...
        'Tag' 'novice' ...
        'String' 'Novice Matlab programmer'}});

  % Install Coma.
  coma('setup', f, ...
       'FactoryDefaults', myapp_factory, ...
       'FileExtension', '.myapp');

  % Install widget call-backs.
  h = getappdata(f, 'coma_handles');
  tem = cellfun('isempty', get(h, {'Callback'}));
  set(h(tem), 'Callback', {@myapp_event});

  % Show figure.
  set(f, 'Visible', 'on');

% Return the factory configuration.
function s = myapp_factory

  s.name = coma('factory', 'edit', '', '');
  s.sex = coma('factory', 'popupmenu', 1, 1);
  s.novice = coma('factory', 'checkbox', 1, 1);

% Widget call-back handler.
function myapp_event(obj, ev)

  % Call-back figure.
  f = coma('figure', obj);

  % Widget handles.
  w = getappdata(f, 'coma_widgets');

  try
    switch get(obj, 'Tag');
     case 'name'
      old = get(obj, 'UserData');
      new = strtrim(get(obj, 'String'));
      if strcmp(old, new) ~= 0
        set(obj, 'UserData', new);
        coma('dirty', f, 1);
      end

     case 'sex'
      old = get(obj, 'UserData');
      new = get(obj, 'Value');
      if old ~= new
        set(obj, 'UserData', new);
        coma('dirty', f, 1);
      end

     case 'novice'
      old = get(obj, 'UserData');
      new = coma('tick', obj);
      if old ~= new
        set(obj, 'UserData', new);
        coma('dirty', f, 1);
      end
    end
  catch
    mtk('error', coma('title', f, 'Error'), lasterr);
  end

%% myapp.m ends here
